//$Id: $
package org.hibernate.ejb.test.emops;

import javax.persistence.EntityManager;

import org.hibernate.ejb.test.TestCase;

/**
 * @author Emmanuel Bernard
 */
public class MergeTest extends TestCase {
	public void testMergeWithIndexColumn() {
		Race race = new Race();
		race.competitors.add( new Competitor() );
		race.competitors.add( new Competitor() );
		race.competitors.add( new Competitor() );
		EntityManager em = factory.createEntityManager();
		em.getTransaction().begin();
		em.persist( race );
		em.flush();
		em.clear();
		race.competitors.add( new Competitor() );
		race.competitors.remove( 2 );
		race = em.merge( race );
		em.flush();
		em.clear();
		race = em.find( Race.class, race.id );
		assertEquals( 3, race.competitors.size() );
		em.getTransaction().rollback();
		em.close();
	}

	public void testRemoveAndMerge() {
		Race race = new Race();
		EntityManager em = factory.createEntityManager();
		em.getTransaction().begin();
		em.persist( race );
		em.flush();
		em.clear();
		race = em.find(Race.class, race.id );
		em.remove(race);
		try {
			race = em.merge( race );
			em.flush();
			fail("Should raise an IllegalArgumentException");
		}
		catch (IllegalArgumentException e) {
			//all good
		}
		catch (Exception e) {
			fail("Should raise an IllegalArgumentException");
		}
		em.getTransaction().rollback();
		em.close();
	}

	public void testConcurrentMerge() {
		Race race = new Race();
		race.name = "Derby";
		EntityManager em = factory.createEntityManager();
		em.getTransaction().begin();
		em.persist( race );
		em.flush();
		em.getTransaction().commit();
		em.close();

		race.name = "Magnicourt";

		em = factory.createEntityManager();
		em.getTransaction().begin();
		Race race2 = em.find(Race.class, race.id );
		race2.name = "Mans";

		race = em.merge( race );
		em.flush();
		em.getTransaction().commit();
		em.close();

		em = factory.createEntityManager();
		em.getTransaction().begin();
		race2 = em.find(Race.class, race.id );
		assertEquals( "Last commit win in merge", "Magnicourt", race2.name );

		em.remove( race2 );
		em.getTransaction().commit();
		em.close();
	}

	public Class[] getAnnotatedClasses() {
		return new Class[] {
				Race.class,
				Competitor.class
		};
	}
}
